@@ -48,14 +48,19 @@ static void update_ime_text(vt_term_t *term, char *preedit_text, char *commit_te
   u_char buf[128];
   size_t len;
 
+  vt_term_set_config(term, "use_local_echo", "false");
+
+#if 0
+  /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
+  ((ui_screen_t*)uiwindow)->is_preediting = 0;
+#endif
+
   if (!utf8_parser && !(utf8_parser = vt_char_encoding_parser_new(VT_UTF8))) {
     return;
   }
 
   (*utf8_parser->init)(utf8_parser);
 
-  vt_term_set_config(term, "use_local_echo", "false");
-
   if (preedit_text) {
     ui_window_t *win;
 
@@ -66,6 +71,11 @@ static void update_ime_text(vt_term_t *term, char *preedit_text, char *commit_te
         return;
       }
 
+#if 0
+      /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
+      ((ui_screen_t*)uiwindow)->is_preediting = 1; /* Hide cursor */
+#endif
+
       vt_term_set_config(term, "use_local_echo", "true");
 
       (*utf8_parser->set_str)(utf8_parser, preedit_text, strlen(preedit_text));
@@ -329,11 +339,12 @@ void Java_mlterm_native_1activity_MLActivity_execCommand(JNIEnv *env, jobject th
   vt_term_t *term;
 
   if ((term = get_current_term()) && term->parser->config_listener) {
-    if (cmd == 4) {
+    if (cmd == 7) {
       connect_to_ssh_server = 1;
-    } else if (((u_int)cmd) <= 3) {
+    } else if (((u_int)cmd) <= 6) {
       char cmd_str4[] = "mlclientx --serv=";
-      char *cmd_str[4] = { "open_pty", "vsplit_screen", "hsplit_screen", cmd_str4 };
+      char *cmd_str[7] = { "open_pty", "vsplit_screen", "hsplit_screen", cmd_str4,
+                           "next_pty", "prev_pty", "close_screen" };
       pthread_mutex_lock(&mutex);
       /* config_listener->exec doesn't modify cmd string unless it contains space character. */
       (*term->parser->config_listener->exec)(term->parser->config_listener->self, cmd_str[cmd]);
