@@ -18,11 +18,19 @@
 #define IM_FCITX_DEBUG 1
 #endif
 
+/*
+ * fcitx doesn't support wayland, so the positioning of gtk-based candidate window of fcitx
+ * doesn't work correctly on wayland.
+ */
 #if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE) || defined(USE_WAYLAND)
 #define KeyPress 2 /* see uitoolkit/fb/ui_display.h */
 #define USE_IM_CANDIDATE_SCREEN
 #endif
 
+#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+#define NO_XKB
+#endif
+
 /* When fcitx encoding is the same as terminal, conv is NULL. */
 #define NEED_TO_CONV(fcitx) ((fcitx)->conv)
 
@@ -100,7 +108,7 @@ static int delete (ui_im_t *im) {
   return ref_count;
 }
 
-#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+#ifdef NO_XKB
 static KeySym native_to_fcitx_ksym(KeySym ksym) {
   switch (ksym) {
     case XK_BackSpace:
@@ -197,27 +205,44 @@ static KeySym native_to_fcitx_ksym(KeySym ksym) {
 
 static int key_event(ui_im_t *im, u_char key_char, KeySym ksym, XKeyEvent *event) {
   im_fcitx_t *fcitx;
+  u_int state;
 
   fcitx = (im_fcitx_t*)im;
 
+#ifdef NO_XKB
+  if (ksym == 0x08 && (event->state & ModMask)) {
+    /* Alt+Backspace => Alt+Control+h (show yourei) */
+    ksym = 'h';
+    state = event->state | ControlMask;
+  } else if (ksym == '[' && (event->state & ControlMask)) {
+    /* Control+[ => Escape (exit yourei) */
+    ksym = XK_Escape;
+    state = event->state & ~ControlMask;
+  } else
+#endif
+  {
+    state = event->state;
+  }
+
   if (event->state & FcitxKeyState_IgnoredMask) {
     /* Is put back in forward_key_event */
     event->state &= ~FcitxKeyState_IgnoredMask;
   } else if (fcitx_client_process_key_sync(
                  fcitx->client, native_to_fcitx_ksym(ksym),
-#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+#ifdef NO_XKB
                  event->keycode,
 #else
                  event->keycode - 8,
 #endif
-                 event->state, event->type == KeyPress ? FCITX_PRESS_KEY : FCITX_RELEASE_KEY,
-#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+                 state, event->type == KeyPress ? FCITX_PRESS_KEY : FCITX_RELEASE_KEY,
+#ifdef NO_XKB
                  0L /* CurrentTime */
 #else
                  event->time
 #endif
                  )) {
     fcitx->is_enabled = TRUE;
+    event->state = state;
     memcpy(&fcitx->prev_key, event, sizeof(XKeyEvent));
 
     g_main_context_iteration(g_main_context_default(), FALSE);
@@ -278,6 +303,8 @@ static void unfocused(ui_im_t *im) {
 
 static void connected(FcitxClient *client, void *data) {
   im_fcitx_t *fcitx;
+  int x;
+  int y;
 
   fcitx = data;
 
@@ -289,6 +316,20 @@ static void connected(FcitxClient *client, void *data) {
 #endif
                             );
   fcitx_client_focus_in(client);
+
+#if 1
+  /*
+   * XXX
+   * To show initial status window (e.g. "Mozc") at the correct position.
+   * Should be moved to enable_im() but "enable-im" event doesn't work. (fcitx 4.2.9.1)
+   */
+  if ((*fcitx->im.listener->get_spot)(fcitx->im.listener->self, NULL, 0, &x, &y)) {
+    u_int line_height;
+
+    line_height = (*fcitx->im.listener->get_line_height)(fcitx->im.listener->self);
+    fcitx_client_set_cursor_rect(fcitx->client, x, y - line_height, 0, line_height);
+  }
+#endif
 }
 
 static void disconnected(FcitxClient *client, void *data) {
@@ -364,7 +405,7 @@ static void forward_key(FcitxClient *client, guint keyval, guint state, gint typ
   fcitx = data;
 
   if (fcitx->prev_key.keycode ==
-#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+#ifdef NO_XKB
       keyval
 #else
       keyval + 8
@@ -492,6 +533,7 @@ static void update_client_side_ui(FcitxClient *client, char *auxup, char *auxdow
 
     (*fcitx->im.listener->get_spot)(fcitx->im.listener->self, fcitx->im.preedit.chars,
                                     fcitx->im.preedit.segment_offset, &x, &y);
+    bl_debug_printf("%d: %d %d\n", fcitx->im.preedit.segment_offset, x, y);
 
     if (fcitx->im.stat_screen == NULL) {
       if (!(fcitx->im.stat_screen = (*syms->ui_im_status_screen_new)(
@@ -509,6 +551,39 @@ static void update_client_side_ui(FcitxClient *client, char *auxup, char *auxdow
       (*fcitx->im.stat_screen->set_spot)(fcitx->im.stat_screen, x, y);
     }
 
+    if (strncmp(candidateword, "1.", 2) == 0) {
+      int count = 2;
+      char *src = candidateword;
+      char *p;
+      char digit[5];
+
+#ifdef __DEBUG
+      bl_msg_printf("Fcitx: %s\n", candidateword);
+#endif
+
+      do {
+        sprintf(digit, " %d.", count);
+
+        if (!(p = strstr(src, digit))) {
+          if (count % 10 == 0 && (src = strstr(src, " 0."))) {
+            count = 1;
+          } else {
+            break;
+          }
+        } else {
+          count++;
+          src = p;
+        }
+
+        *src = '\n';
+        src += strlen(digit);
+      } while (count < 100);
+
+#ifdef __DEBUG
+      bl_msg_printf("%s\n", candidateword);
+#endif
+    }
+
     if (NEED_TO_CONV(fcitx)) {
       (*parser_utf8->init)(parser_utf8);
       if (im_convert_encoding(parser_utf8, fcitx->conv, candidateword, &tmp,
@@ -602,9 +677,7 @@ static void update_formatted_preedit(FcitxClient *client, GPtrArray *list, int c
           bg_color = VT_BG_COLOR;
         }
 
-        if ((*syms->vt_convert_to_internal_ch)(
-                &ch, (*fcitx->im.listener->get_unicode_policy)(fcitx->im.listener->self),
-                US_ASCII) <= 0) {
+        if ((*syms->vt_convert_to_internal_ch)(fcitx->im.vtparser, &ch) <= 0) {
           continue;
         }
 
@@ -651,9 +724,7 @@ static void update_formatted_preedit(FcitxClient *client, GPtrArray *list, int c
 
 #endif
 
-#ifndef USE_WAYLAND
 static void connection_handler(void) { g_main_context_iteration(g_main_context_default(), FALSE); }
-#endif
 
 /* --- global functions --- */
 
@@ -735,9 +806,7 @@ ui_im_t *im_fcitx_new(u_int64_t magic, vt_char_encoding_t term_encoding,
   fcitx->im.unfocused = unfocused;
 
   if (ref_count++ == 0) {
-#ifndef USE_WAYLAND
     (*syms->ui_event_source_add_fd)(FCITX_ID, connection_handler);
-#endif
 
     if (!(parser_utf8 = (*syms->vt_char_encoding_parser_new)(VT_UTF8))) {
       goto error;
