@@ -94,7 +94,7 @@ static void show_config(void *p, char *msg) {
 
 #ifdef USE_LIBSSH2
 static JNIEnv *env_for_dialog;
-static int dialog_callback(bl_dialog_style_t style, char *msg) {
+static int dialog_callback(bl_dialog_style_t style, const char *msg) {
   jclass class;
   static jmethodID mid;
 
@@ -461,7 +461,7 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
   if (!str_parser) {
     bl_conf_t *conf;
 
-    bl_init_prog("mlterm", "3.8.2");
+    bl_init_prog("mlterm", "3.8.3");
     bl_set_sys_conf_dir(CONFIG_PATH);
     bl_locale_init("");
     bl_sig_child_init();
@@ -697,7 +697,8 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
             logging_vt_seq, unicode_policy, col_size_a, use_char_combining, use_multi_col_char,
             0 /* use_ctl */, 0 /* bidi_mode */, NULL /* bidi_separators */,
             0 /* use_dynamic_comb */, BSM_STATIC, 0 /* vertical_mode */, use_local_echo, NULL, NULL,
-            0 /* alt_color_mode */, 0 /* use_ot_layout */, 0 /* cursor_style */))) {
+            0 /* alt_color_mode */, 0 /* use_ot_layout */, 0 /* cursor_style */,
+            0 /* ignore_broadcasted_chars */))) {
     goto error;
   }
 
