@@ -78,12 +78,12 @@ static char *get_font_name_list_for_fontset(ui_font_cache_t *font_cache) {
              TYPE_XCORE, font_cache->font_config->font_present & ~FONT_AA)) == NULL) {
       font_name_list = NULL;
     } else {
-      font_name_list = ui_get_all_config_font_names(font_config, font_cache->font_size);
+      font_name_list = ui_get_config_font_names_all(font_config, font_cache->font_size);
 
       ui_release_font_config(font_config);
     }
   } else {
-    font_name_list = ui_get_all_config_font_names(font_cache->font_config, font_cache->font_size);
+    font_name_list = ui_get_config_font_names_all(font_cache->font_config, font_cache->font_size);
   }
 
   if (font_name_list) {
@@ -109,7 +109,13 @@ static char *get_font_name_list_for_fontset(ui_font_cache_t *font_cache) {
 
 /* --- global functions --- */
 
-void ui_set_use_leftward_double_drawing(int use) { leftward_double_drawing = use; }
+void ui_set_use_leftward_double_drawing(int use) {
+  if (num_of_caches > 0) {
+    bl_msg_printf("Unable to change leftward_double_drawing option.\n");
+  } else {
+    leftward_double_drawing = use;
+  }
+}
 
 ui_font_cache_t *ui_acquire_font_cache(Display *display, u_int font_size,
                                        ef_charset_t usascii_font_cs, ui_font_config_t *font_config,
@@ -322,7 +328,7 @@ ui_font_t *ui_font_cache_get_xfont(ui_font_cache_t *font_cache, vt_font_t font)
 found:
   size_attr = SIZE_ATTR_OF(font);
 
-  if ((xfont = ui_font_new(font_cache->display, FONT_WITHOUT_SIZE_ATTR(font), size_attr,
+  if ((xfont = ui_font_new(font_cache->display, NO_SIZE_ATTR(font), size_attr,
                            font_cache->font_config->type_engine, font_present, fontname,
                            font_cache->font_size, col_width, use_medium_for_bold,
                            font_cache->letter_space)) ||
