@@ -8,7 +8,8 @@
 #include <pobl/bl_locale.h> /* bl_get_lang */
 #include <pobl/bl_args.h>
 
-#include "vt_term_manager.h"
+#include <vt_term_manager.h>
+#include "ui_selection_encoding.h"
 
 #if defined(__ANDROID__) || defined(USE_QUARTZ) || defined(USE_WIN32API)
 /* Shrink unused memory */
@@ -30,9 +31,7 @@ static vt_char_encoding_t get_encoding(const char *value,
     return encoding;
   }
 
-  if ((encoding = vt_get_char_encoding("auto")) == VT_UNKNOWN_ENCODING) {
-    encoding = VT_ISO8859_1;
-  }
+  encoding = vt_get_char_encoding("auto");
 
   if (is_auto_encoding) {
     *is_auto_encoding = 1;
@@ -46,23 +45,6 @@ static vt_char_encoding_t get_encoding(const char *value,
 int ui_prepare_for_main_config(bl_conf_t *conf) {
   char *rcpath;
 
-#ifdef ENABLE_BACKWARD_COMPAT
-  /*
-   * XXX
-   * "mlterm/core" is for backward compatibility with 1.9.44
-   */
-
-  if ((rcpath = bl_get_sys_rc_path("mlterm/core"))) {
-    bl_conf_read(conf, rcpath);
-    free(rcpath);
-  }
-
-  if ((rcpath = bl_get_user_rc_path("mlterm/core"))) {
-    bl_conf_read(conf, rcpath);
-    free(rcpath);
-  }
-#endif
-
   if ((rcpath = bl_get_sys_rc_path("mlterm/main"))) {
     bl_conf_read(conf, rcpath);
     free(rcpath);
@@ -94,8 +76,10 @@ int ui_prepare_for_main_config(bl_conf_t *conf) {
 
   bl_conf_add_opt(conf, '1', "wscr", 0, "screen_width_ratio",
                   "screen width in percent against font width [100]");
+#if 1 /* BACKWARD COMPAT (3.8.2 or before) */
   bl_conf_add_opt(conf, '2', "hscr", 0, "screen_height_ratio",
                   "screen height in percent against font height [100]");
+#endif
 #ifndef NO_IMAGE
   bl_conf_add_opt(conf, '3', "contrast", 0, "contrast",
                   "contrast of background image in percent [100]");
@@ -139,7 +123,7 @@ int ui_prepare_for_main_config(bl_conf_t *conf) {
                   "input method (xim/kbd/uim/m17nlib/scim/ibus/fcitx/canna/wnn/skk/iiimf/none) [xim]");
   bl_conf_add_opt(conf, 'N', "name", 0, "app_name", "application name");
   bl_conf_add_opt(conf, 'O', "sbmod", 0, "scrollbar_mode",
-                  "scrollbar mode (none/left/right) [none]");
+                  "scrollbar mode (none/left/right/autohide) [none]");
   bl_conf_add_opt(conf, 'P', "clip", 1, "use_clipboard",
                   "use CLIPBOARD (not only PRIMARY) selection [true]");
   bl_conf_add_opt(conf, 'Q', "vcur", 1, "use_vertical_cursor",
@@ -320,6 +304,12 @@ int ui_prepare_for_main_config(bl_conf_t *conf) {
   bl_conf_add_opt(conf, '\0', "slp", 1, "start_with_local_pty",
                   "start mlterm with local pty instead of ssh connection [false]");
 #endif
+  bl_conf_add_opt(conf, '\0', "trim", 1, "trim_trailing_newline_in_pasting",
+                  "trim trailing newline in pasting text. [false]");
+  bl_conf_add_opt(conf, '\0', "bc", 1, "broadcast",
+                  "broadcast input characters to all ptys [false]");
+  bl_conf_add_opt(conf, '\0', "ibc", 1, "ignore_broadcasted_chars",
+                  "ignore broadcasted characters [false]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -588,6 +578,16 @@ int ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int argc
 
   main_config->screen_width_ratio = 100;
 
+#if 1 /* BACKWARD COMPAT (3.8.2 or before) */
+  if ((value = bl_conf_get_value(conf, "screen_height_ratio"))) {
+    u_int ratio;
+
+    if (bl_str_to_uint(&ratio, value) && ratio) {
+      main_config->screen_width_ratio = ratio;
+    }
+  }
+#endif
+
   if ((value = bl_conf_get_value(conf, "screen_width_ratio"))) {
     u_int ratio;
 
@@ -598,18 +598,6 @@ int ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int argc
     }
   }
 
-  main_config->screen_height_ratio = 100;
-
-  if ((value = bl_conf_get_value(conf, "screen_height_ratio"))) {
-    u_int ratio;
-
-    if (bl_str_to_uint(&ratio, value) && ratio) {
-      main_config->screen_height_ratio = ratio;
-    } else {
-      bl_msg_printf(invalid_msg, "screen_height_ratio", value);
-    }
-  }
-
   main_config->use_multi_col_char = 1;
 
   if ((value = bl_conf_get_value(conf, "use_multi_column_char"))) {
@@ -678,9 +666,7 @@ int ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int argc
   }
 
   if ((value = bl_conf_get_value(conf, "big5_buggy"))) {
-    if (strcmp(value, "true") == 0) {
-      main_config->big5_buggy = 1;
-    }
+    ui_set_big5_selection_buggy(strcmp(value, "true") == 0);
   }
 
   main_config->use_mdi = 1;
@@ -1317,6 +1303,24 @@ int ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int argc
   }
 #endif
 
+  if ((value = bl_conf_get_value(conf, "trim_trailing_newline_in_pasting"))) {
+    if (strcmp(value, "true") == 0) {
+      ui_set_trim_trailing_newline_in_pasting(1);
+    }
+  }
+
+  if ((value = bl_conf_get_value(conf, "broadcast"))) {
+    if (strcmp(value, "true") == 0) {
+      vt_set_broadcasting(1);
+    }
+  }
+
+  if ((value = bl_conf_get_value(conf, "ignore_broadcasted_chars"))) {
+    if (strcmp(value, "true") == 0) {
+      main_config->ignore_broadcasted_chars = 1;
+    }
+  }
+
 #ifdef USE_IM_CURSOR_COLOR
   if ((value = bl_conf_get_value(conf, "im_cursor_color"))) {
     if (*value) {
