@@ -380,10 +380,10 @@ static void keyboard_enter(void *data, struct wl_keyboard *keyboard,
   ui_wlserv_t *wlserv = data;
   ui_window_t *win;
 
-  wlserv->current_kbd_surface = surface;
-
 #ifdef COMPAT_LIBVTE
   if (!disp || disp->num_of_roots == 0) {
+    wlserv->current_kbd_surface = surface;
+
     return;
   }
 #endif
@@ -394,6 +394,8 @@ static void keyboard_enter(void *data, struct wl_keyboard *keyboard,
     surface = disp->display->surface;
   }
 
+  wlserv->current_kbd_surface = surface;
+
 #ifdef __DEBUG
   bl_debug_printf("KBD ENTER %p\n", surface);
 #endif
@@ -423,16 +425,39 @@ static void keyboard_leave(void *data, struct wl_keyboard *keyboard,
   wlserv->prev_kev.type = 0;
 #endif
 
+#ifdef COMPAT_LIBVTE
+  if (!disp || disp->num_of_roots == 0) {
+    return;
+  }
+#endif
+
   if (wlserv->current_kbd_surface == surface) {
+    u_int count;
+
+    for (count = 0; count < num_of_displays; count++) {
+      if (displays[count]->display->parent == disp) {
+        /*
+         * Don't send FocusOut event to the surface which has an input method surface
+         * as transient one, because the surface might receive this event by creating
+         * and showing the input method surface.
+         * As a secondary effect, the surface can't receive FocusOut event as long as
+         * the input method surface is active.
+         */
+        return;
+      }
+    }
+
     wlserv->current_kbd_surface = NULL;
   }
 
   /* surface may have been already destroyed. So null check of disp is necessary. */
-  if (disp && (win = search_focused_window(disp->roots[0]))) {
+  if (disp && !disp->display->parent /* not input method window */ &&
+      (win = search_focused_window(disp->roots[0]))) {
+    XEvent ev;
+
 #ifdef __DEBUG
     bl_debug_printf("UNFOCUSED %p\n", win);
 #endif
-    XEvent ev;
     ev.type = FocusOut;
     ui_window_receive_event(ui_get_root_window(win), &ev);
   }
@@ -492,7 +517,7 @@ static void keyboard_modifiers(void *data, struct wl_keyboard *keyboard,
   if (mod_mask & (1 << wlserv->xkb->ctrl))
     wlserv->xkb->mods |= ControlMask;
   if (mod_mask & (1 << wlserv->xkb->alt))
-    wlserv->xkb->mods |= ModMask;
+    wlserv->xkb->mods |= Mod1Mask; /* XXX ModMask disables yourei by Ctrl+Alt+H on fcitx */
   if (mod_mask & (1 << wlserv->xkb->shift))
     wlserv->xkb->mods |= ShiftMask;
 #if 0
@@ -728,11 +753,13 @@ static void pointer_button(void *data, struct wl_pointer *pointer, uint32_t seri
       } else {
         return;
       }
+
+      wlserv->pointer_button = ev.button;
     } else {
       ev.type = ButtonRelease;
-      ev.button = 0;
+      ev.button = wlserv->pointer_button;
+      wlserv->pointer_button = 0;
     }
-    wlserv->pointer_button = ev.button;
     ev.time = time;
     ev.state = wlserv->xkb->mods;
 
@@ -761,7 +788,9 @@ static void pointer_button(void *data, struct wl_pointer *pointer, uint32_t seri
     ui_window_receive_event(win, &ev);
 
 #ifdef COMPAT_LIBVTE
-    focus_gtk_window(win, time);
+    if (ev.type == ButtonPress && disp->display->parent == NULL /* Not input method */) {
+      focus_gtk_window(win, time);
+    }
 #endif
   }
 }
@@ -1272,6 +1301,11 @@ static void data_source_send(void *data, struct wl_data_source *source, const ch
   }
 }
 
+#ifdef COMPAT_LIBVTE
+#include <glib.h>
+gint64 deadline_ignoring_source_cancelled_event;
+#endif
+
 static void data_source_cancelled(void *data, struct wl_data_source *source) {
   ui_display_t *disp = data;
   ui_wlserv_t *wlserv = disp->display->wlserv;
@@ -1280,9 +1314,18 @@ static void data_source_cancelled(void *data, struct wl_data_source *source) {
   bl_debug_printf("SOURCE CANCEL %p %p\n", source, wlserv->sel_source);
 #endif
 
-  if (source == wlserv->sel_source) {
-    ui_display_clear_selection(disp, disp->selection_owner);
+#ifdef COMPAT_LIBVTE
+  if (deadline_ignoring_source_cancelled_event != 0 &&
+      deadline_ignoring_source_cancelled_event > g_get_monotonic_time()) {
+  } else {
+    deadline_ignoring_source_cancelled_event = 0;
+#endif
+    if (source == wlserv->sel_source) {
+      ui_display_clear_selection(disp, disp->selection_owner);
+    }
+#ifdef COMPAT_LIBVTE
   }
+#endif
 }
 
 void data_source_dnd_drop_performed(void *data, struct wl_data_source *wl_data_source) {
@@ -1527,6 +1570,15 @@ static void damage_buffer(Display *display, int x, int y, u_int width, u_int hei
 static int flush_display(Display *display) {
   if (flush_damage(display)) {
     wl_surface_commit(display->surface);
+#ifdef COMPAT_LIBVTE
+    if (display->parent) {
+      /*
+       * Input method window.
+       * refrect wl_surface_damage() immediately.
+       */
+      wl_surface_commit(display->parent_surface);
+    }
+#endif
 
     return 1;
   } else {
@@ -1534,8 +1586,7 @@ static int flush_display(Display *display) {
   }
 }
 
-static void create_surface(ui_display_t *disp, int x, int y, u_int width, u_int height,
-                           char *app_name) {
+static void create_surface(ui_display_t *disp, u_int width, u_int height, char *app_name) {
   Display *display = disp->display;
   ui_wlserv_t *wlserv = display->wlserv;
 
@@ -1548,14 +1599,9 @@ static void create_surface(ui_display_t *disp, int x, int y, u_int width, u_int
   display->shell_surface = wl_shell_get_shell_surface(wlserv->shell, display->surface);
   wl_shell_surface_set_class(display->shell_surface, app_name);
 
-  if (display->parent) {
-#ifdef __DEBUG
-    bl_debug_printf("Move display (set transient) at %d %d\n", x, y);
-#endif
-    wl_shell_surface_set_transient(display->shell_surface,
-                                   display->parent->display->surface, x, y,
-                                   WL_SHELL_SURFACE_TRANSIENT_INACTIVE);
-  } else {
+  if (!display->parent) {
+    /* Not input method */
+
     wl_surface_add_listener(display->surface, &surface_listener, wlserv);
     wl_shell_surface_add_listener(display->shell_surface, &shell_surface_listener, disp);
     wl_shell_surface_set_toplevel(display->shell_surface);
@@ -1733,7 +1779,7 @@ int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, in
    */
   disp->roots[disp->num_of_roots++] = root;
 
-  create_surface(disp, x, y, ACTUAL_WIDTH(root), ACTUAL_HEIGHT(root), root->app_name);
+  create_surface(disp, ACTUAL_WIDTH(root), ACTUAL_HEIGHT(root), root->app_name);
 
 #ifdef COMPAT_LIBVTE
   if (!disp->display->parent) {
@@ -1742,6 +1788,26 @@ int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, in
 #endif
   {
     create_shm_buffer(disp->display);
+
+#ifndef COMPAT_LIBVTE
+    if (disp->display->parent)
+#endif
+    {
+      /* XXX Input Method */
+
+      /* XXX
+       * x and y of wl_shell_surface_set_transient() aren't applied without calling
+       * wl_surface_commit() here before calling wl_shell_surface_set_transient().
+       */
+      wl_surface_commit(disp->display->surface);
+
+      /*
+       * XXX wl_shell_surface_set_transient() doesn't move surface to the specified position
+       * if it is called before create_shm_buffer().
+       */
+      ui_display_move(disp, x, y);
+    }
+
     ui_window_show(root, hint);
   }
 
@@ -1855,6 +1921,11 @@ void ui_display_sync(ui_display_t *disp) {
   u_int count;
   int flushed = 0;
 
+  /*
+   * ui_display_sync() is called from ui_display_idling() alone on libvte compat library.
+   * ui_display_idling() is called with disp in vte.c, not displays in ui_display.c, so
+   * call flush_display() with all displays here.
+   */
   for (count = 0; count < num_of_displays; count++) {
     if (displays[count]->display->buffer) {
       flushed |= flush_display(displays[count]->display);
@@ -2071,6 +2142,12 @@ void ui_display_copy_lines(ui_display_t *disp, int src_x, int src_y, int dst_x,
   u_int count;
   size_t line_length;
 
+#ifdef COMPAT_LIBVTE
+  if (!display->buffer) {
+    return;
+  }
+#endif
+
   if (rotate_display) {
     int tmp;
 
