@@ -31,6 +31,7 @@
 
 #define DIVIDE_ROUNDING(a, b) (((int)((a)*10 + (b)*5)) / ((int)((b)*10)))
 #define DIVIDE_ROUNDINGUP(a, b) (((int)((a)*10 + (b)*10 - 1)) / ((int)((b)*10)))
+#define DIVIDE_ROUNDINGDOWN(a, b) (((int)(a)*10) / ((int)((b)*10)))
 
 #ifdef WORDS_BIGENDIAN
 #define _TOINT32(p, is_be) ((is_be) ? TOINT32(p) : LE32DEC(p))
@@ -566,7 +567,9 @@ static int is_pcf(const char *file_path) {
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
+#ifdef FT_LCD_FILTER_H
 #include FT_LCD_FILTER_H
+#endif
 
 /* 0 - 511 */
 #define SEG(idx) (((idx) >> 7) & 0x1ff)
@@ -645,7 +648,8 @@ static int load_glyph(FT_Face face, int32_t format, u_int32_t code, int is_aa) {
   return 1;
 }
 
-static int load_ft(XFontStruct *xfont, const char *file_path, int32_t format, int is_aa) {
+static int load_ft(XFontStruct *xfont, const char *file_path, int32_t format, int is_aa,
+                   u_int force_height) {
   u_int count;
   FT_Face face;
   u_int fontsize;
@@ -659,7 +663,9 @@ static int load_ft(XFontStruct *xfont, const char *file_path, int32_t format, in
       return 0;
     }
 
+#ifdef FT_LCD_FILTER_H
     FT_Library_SetLcdFilter(library, FT_LCD_FILTER_DEFAULT);
+#endif
   }
 
   fontsize = (format & ~(FONT_BOLD | FONT_ITALIC));
@@ -688,7 +694,7 @@ face_found:
   xfont->face = face;
   xfont->is_aa = is_aa;
 
-  if (!load_glyph(face, format, get_glyph_index(face, 'W'), is_aa)) {
+  if (!load_glyph(face, format, get_glyph_index(face, 'M'), is_aa)) {
     bl_msg_printf("%s doesn't have outline glyphs.\n", file_path);
 
     goto error;
@@ -720,8 +726,12 @@ face_found:
     xfont->width = face->glyph->bitmap.width;
   }
 
-  xfont->height = (face->max_advance_height * face->size->metrics.y_ppem + face->units_per_EM - 1) /
-                  face->units_per_EM;
+  if (force_height) {
+    xfont->height = force_height;
+  } else {
+    xfont->height = (face->max_advance_height * face->size->metrics.y_ppem +
+                     face->units_per_EM - 1) / face->units_per_EM;
+  }
 #ifdef __DEBUG
   bl_debug_printf("maxh %d ppem %d units %d => h %d\n",
                   face->max_advance_height, face->size->metrics.y_ppem,
@@ -992,7 +1002,7 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
 static int load_xfont(XFontStruct *xfont, const char *file_path, int32_t format,
                       u_int bytes_per_pixel, ef_charset_t cs, int noaa) {
   if (!is_pcf(file_path)) {
-    return load_ft(xfont, file_path, format, (bytes_per_pixel > 1) && !noaa);
+    return load_ft(xfont, file_path, format, (bytes_per_pixel > 1) && !noaa, 0);
   } else {
     return load_pcf(xfont, file_path);
   }
@@ -1052,6 +1062,10 @@ static void compl_xfonts_delete(XFontStruct **xfonts) {
 
 /* Same processing as win32/ui_font.c (partially) and libtype/ui_font_ft.c */
 static int parse_fc_font_name(char **font_family,
+                              int *is_bold,   /* if bold is not specified in font_name,
+                                                 not changed. */
+                              int *is_italic, /* if italic is not specified in font_name,
+                                                 not changed. */
                               u_int *percent, /* if percent is not specified in font_name,
                                                  not changed. */
                               char *font_name /* modified by this function. */ ) {
@@ -1124,62 +1138,28 @@ static int parse_fc_font_name(char **font_family,
         break;
       } else {
         int count;
-        struct {
-          char *style;
-          int weight;
-          int slant;
-
-        } styles[] = {
-            /*
-             * Portable styles.
-             */
-            /* slant */
-            {
-             "italic", 0, FC_SLANT_ITALIC,
-            },
-            /* weight */
-            {
-             "bold", FC_WEIGHT_BOLD, 0,
-            },
-
-            /*
-             * Hack for styles which can be returned by
-             * gtk_font_selection_dialog_get_font_name().
-             */
-            /* slant */
-            {
-             "oblique", 0, FC_SLANT_OBLIQUE,
-            },
-            /* weight */
-            {
-             "light", /* e.g. "Bookman Old Style Light" */
-             FC_WEIGHT_LIGHT, 0,
-            },
-            {
-             "semi-bold", FC_WEIGHT_DEMIBOLD, 0,
-            },
-            {
-             "heavy", /* e.g. "Arial Black Heavy" */
-             FC_WEIGHT_BLACK, 0,
-            },
-            /* other */
-            {
-             "semi-condensed", /* XXX This style is ignored. */
-             0, 0,
-            },
-        };
+        char *styles[] = { "italic",  "bold", "oblique", "light", "semi-bold", "heavy",
+                           "semi-condensed", };
 
         for (count = 0; count < sizeof(styles) / sizeof(styles[0]); count++) {
           size_t len_v;
 
-          len_v = strlen(styles[count].style);
+          len_v = strlen(styles[count]);
 
           /* XXX strncasecmp is not portable? */
-          if (len >= len_v && strncasecmp(p, styles[count].style, len_v) == 0) {
+          if (len >= len_v && strncasecmp(p, styles[count], len_v) == 0) {
             /* [WEIGHT] [SLANT] */
             *orig_p = '\0';
             step = len_v;
 
+            if (count <= 1) {
+              if (count == 0) {
+                *is_italic = 1;
+              } else {
+                *is_bold = 1;
+              }
+            }
+
             goto next_char;
           }
         }
@@ -1287,18 +1267,21 @@ static int check_iscii_font(FcPattern *pattern) {
   }
 }
 
-static FcPattern *parse_font_name(const char *fontname, u_int *percent, ef_charset_t cs) {
+static FcPattern *parse_font_name(const char *fontname, int *is_bold, int *is_italic,
+                                  u_int *percent, ef_charset_t cs) {
   FcPattern *pattern;
   FcPattern *match;
   FcResult result;
   char *family;
 
   *percent = 0;
+  *is_bold = 0;
+  *is_italic = 0;
 
   if (!fontname) {
     family = NULL;
   } else {
-    parse_fc_font_name(&family, percent, bl_str_alloca_dup(fontname));
+    parse_fc_font_name(&family, is_bold, is_italic, percent, bl_str_alloca_dup(fontname));
   }
 
   if ((pattern = fc_pattern_create(family))) {
@@ -1411,7 +1394,15 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
         if (!(compl = get_cached_xfont(fc_files[count], xfont->format))) {
           if (!(compl = calloc(1, sizeof(XFontStruct)))) {
             continue;
-          } else if (!load_ft(compl, fc_files[count], xfont->format, xfont->is_aa)) {
+          }
+          /*
+           * XXX
+           * If xfont->height is 17 and compl->height is 15, garbage is left in drawing glyphs
+           * by compl.
+           * force_height (xfont->height) forcibly changes the height of the font from natural one
+           * to the same one as xfont->height.
+           */
+          else if (!load_ft(compl, fc_files[count], xfont->format, xfont->is_aa, xfont->height)) {
             free(compl);
             continue;
           }
@@ -1549,6 +1540,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
   char *decsp_id = NULL;
   u_int percent;
   ui_font_t *font;
+  vt_font_t orig_id = id;
 #ifdef USE_FREETYPE
   u_int format;
 #endif
@@ -1559,11 +1551,20 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
   if (use_fontconfig) {
     FcPattern *pattern;
     FcValue val;
+    int is_bold;
+    int is_italic;
 
-    if (!(pattern = parse_font_name(fontname, &percent, cs))) {
+    if (!(pattern = parse_font_name(fontname, &is_bold, &is_italic, &percent, cs))) {
       return NULL;
     }
 
+    if (is_bold) {
+      id |= FONT_BOLD;
+    }
+    if (is_italic) {
+      id |= FONT_ITALIC;
+    }
+
     if (FcPatternGet(pattern, FC_FILE, 0, &val) != FcResultMatch) {
       FcPatternDestroy(pattern);
 
@@ -1704,14 +1705,10 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
 #ifdef USE_FREETYPE
 #ifdef USE_FONTCONFIG
   if (use_fontconfig && dpi_for_fc > 0.0) {
-    fontsize = DIVIDE_ROUNDINGUP(dpi_for_fc * fontsize, 72);
+    fontsize = DIVIDE_ROUNDINGDOWN(dpi_for_fc * fontsize, 72);
   }
 #endif
-  if (percent > 0) {
-    format = DIVIDE_ROUNDING(fontsize * percent, 100) | (id & (FONT_BOLD | FONT_ITALIC));
-  } else {
-    format = fontsize | (id & (FONT_BOLD | FONT_ITALIC));
-  }
+  format = fontsize | (id & (FONT_BOLD | FONT_ITALIC));
 #endif
 
   if (decsp_id) {
@@ -1782,7 +1779,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
 xfont_loaded:
   /* Following is almost the same processing as xlib. */
 
-  font->id = id;
+  font->id = orig_id;
 
   if (font->id & FONT_FULLWIDTH) {
     font->cols = 2;
