@@ -1668,7 +1668,6 @@ static int reconnect(vt_pty_ssh_t *pty) {
   }
 
   orig_pty = *pty;
-  memset(pty, 0, sizeof(*pty));
   pty->session = session;
   if (!open_channel(pty, session->stored->cmd_path, session->stored->argv, session->stored->env,
                     session->stored->cols, session->stored->rows, session->stored->width_pix,
@@ -1677,9 +1676,7 @@ static int reconnect(vt_pty_ssh_t *pty) {
     return 0;
   }
 
-  /* XXX See vt_pty_delete() */
-  free(orig_pty.pty.buf);
-  free(orig_pty.pty.cmd_line);
+  free(orig_pty.pty.cmd_line); /* newly allocated in open_channel() */
   final(&orig_pty.pty);
 
   return 1;
@@ -1851,6 +1848,17 @@ static void save_data_for_reconnect(ssh_session_t *session, const char *cmd_path
   }
 }
 
+static char *get_user_name(void) {
+  char *user;
+
+  /* USER/LOGNAME: unix , USERNAME: win32 */
+  if (!(user = getenv("USER")) && !(user = getenv("USERNAME"))) {
+    user = getenv("LOGNAME");
+  }
+
+  return user;
+}
+
 /* --- global functions --- */
 
 /*
@@ -1872,8 +1880,7 @@ vt_pty_t *vt_pty_ssh_new(const char *cmd_path, /* can be NULL */
     return NULL;
   }
 
-  /* USER: unix , USERNAME: win32 */
-  if (!user && !(user = getenv("USER")) && !(user = getenv("USERNAME"))) {
+  if (!user && !(user = get_user_name())) {
     return NULL;
   }
 
@@ -1906,8 +1913,7 @@ void *vt_search_ssh_session(const char *host, const char *port, /* can be NULL *
                             ) {
   int count;
 
-  /* USER: unix , USERNAME: win32 */
-  if (!user && !(user = getenv("USER")) && !(user = getenv("USERNAME"))) {
+  if (!user && !(user = get_user_name())) {
     return NULL;
   }
 
