@@ -21,19 +21,19 @@
                          * (total 20 bytes)
                          */
 #define XCT_MAX_SIZE 20
-#define MLCHAR_UTF_MAX_SIZE (UTF_MAX_SIZE * (MAX_COMB_SIZE + 1))
-#define MLCHAR_XCT_MAX_SIZE (XCT_MAX_SIZE * (MAX_COMB_SIZE + 1))
+#define VTCHAR_UTF_MAX_SIZE (UTF_MAX_SIZE * (MAX_COMB_SIZE + 1))
+#define VTCHAR_XCT_MAX_SIZE (XCT_MAX_SIZE * (MAX_COMB_SIZE + 1))
 
 /* For inline pictures (see x_picture.c) */
 #define PICTURE_CHARSET 0x1ff
 #define PICTURE_ID_BITS 9   /* fg or bg color */
-#define PICTURE_POS_BITS 23 /* code */
+#define PICTURE_POS_BITS 23 /* code (Can be shrunk to 21 bits) */
 
-enum {
+typedef enum {
   UNDERLINE_NONE,
   UNDERLINE_NORMAL,
   UNDERLINE_DOUBLE,
-};
+} vt_underline_style_t;
 
 /*
  * This object size should be kept as small as possible.
@@ -71,7 +71,7 @@ typedef struct vt_char {
  * 1 bit : is_single_ch(0 or 1)
  */
 #ifdef WORDS_BIGENDIAN
-      u_int code : 23;
+      u_int code : 23; /* Can be shrunk to 21 bits (Unicode: 0-10FFFF) */
       u_int fg_color : 9;
       u_int bg_color : 9;
       u_int attr : 23;
@@ -79,7 +79,7 @@ typedef struct vt_char {
       u_int attr : 23;
       u_int fg_color : 9;
       u_int bg_color : 9;
-      u_int code : 23;
+      u_int code : 23; /* Can be shrunk to 21 bits (Unicode: 0-10FFFF) */
 #endif
     } ch;
 
@@ -95,6 +95,8 @@ typedef struct vt_char {
 
 int vt_set_use_multi_col_char(int use_it);
 
+int vt_get_unicode_area(vt_font_t font, int *min, int *max);
+
 vt_font_t vt_get_unicode_area_font(u_int32_t min, u_int32_t max);
 
 void vt_set_blink_chars_visible(int visible);
