@@ -232,7 +232,7 @@ vt_term_t *vt_term_new(const char *term_type, u_int cols, u_int rows, u_int tab_
                        int use_dynamic_comb, vt_bs_mode_t bs_mode, vt_vertical_mode_t vertical_mode,
                        int use_local_echo, const char *win_name, const char *icon_name,
                        vt_alt_color_mode_t alt_color_mode, int use_ot_layout,
-                       vt_cursor_style_t cursor_style) {
+                       vt_cursor_style_t cursor_style, int ignore_broadcasted_chars) {
   vt_termcap_ptr_t termcap;
   vt_term_t *term;
 
@@ -257,17 +257,21 @@ vt_term_t *vt_term_new(const char *term_type, u_int cols, u_int rows, u_int tab_
     goto error;
   }
 
+  term->use_ot_layout = use_ot_layout;
+
+#ifndef NOT_CONVERT_TO_ISCII
 #ifdef USE_HARFBUZZ
-  if (!(term->use_ot_layout = use_ot_layout))
+  if (!term->use_ot_layout)
 #endif
   {
     policy |= CONVERT_UNICODE_TO_ISCII;
   }
+#endif
 
   if (!(term->parser = vt_parser_new(term->screen, termcap, encoding, is_auto_encoding,
                                      use_auto_detect, logging_vt_seq, policy, col_size_a,
                                      use_char_combining, use_multi_col_char, win_name, icon_name,
-                                     alt_color_mode, cursor_style))) {
+                                     alt_color_mode, cursor_style, ignore_broadcasted_chars))) {
 #ifdef DEBUG
     bl_warn_printf(BL_DEBUG_TAG " vt_parser_new failed.\n");
 #endif
@@ -348,17 +352,10 @@ int vt_term_zombie(vt_term_t *term) {
   return 1;
 }
 
+/* The caller should swap width_pix and height_pix in vertical mode. */
 int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **env,
                      const char *host, const char *work_dir, const char *pass, const char *pubkey,
                      const char *privkey, u_int width_pix, u_int height_pix) {
-  if (term->vertical_mode) {
-    u_int tmp;
-
-    tmp = width_pix;
-    width_pix = height_pix;
-    height_pix = tmp;
-  }
-
   if (!term->pty) {
 #ifdef OPEN_PTY_ASYNC
     char *user;
@@ -486,7 +483,7 @@ int vt_term_detach(vt_term_t *term) {
 }
 
 void vt_term_set_use_ot_layout(vt_term_t *term, int flag) {
-#ifdef USE_HARFBUZZ
+#if defined(USE_HARFBUZZ) && !defined(NOT_CONVERT_TO_ISCII)
   vt_unicode_policy_t policy;
 
   policy = vt_parser_get_unicode_policy(term->parser);
@@ -555,6 +552,7 @@ size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len) {
   return vt_write_to_pty(term->pty, buf, len);
 }
 
+/* The caller should swap width_pix and height_pix in vertical mode. */
 int vt_term_resize(vt_term_t *term, u_int cols, u_int rows, u_int width_pix, u_int height_pix) {
   if (term->pty) {
     vt_set_pty_winsize(term->pty, cols, rows, width_pix, height_pix);
@@ -592,7 +590,7 @@ int vt_term_unhighlight_cursor(vt_term_t *term, int revert_visual) {
   }
 
   if (revert_visual) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
@@ -636,7 +634,7 @@ int vt_term_set_modified_region_in_screen(vt_term_t *term, int beg_char_index, i
   }
 
   if (revert_to_visual) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
@@ -666,7 +664,7 @@ int vt_term_set_modified_lines(vt_term_t *term, int beg, int end) {
   }
 
   if (revert_to_visual) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
@@ -695,7 +693,7 @@ int vt_term_set_modified_lines_in_screen(vt_term_t *term, int beg, int end) {
   }
 
   if (revert_to_visual) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
@@ -718,7 +716,7 @@ int vt_term_set_modified_all_lines_in_screen(vt_term_t *term) {
   vt_screen_set_modified_all(term->screen);
 
   if (revert_to_visual) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
@@ -748,7 +746,7 @@ int vt_term_updated_all(vt_term_t *term) {
   }
 
   if (!vt_screen_logical_visual_is_reversible(term->screen)) {
-    /* vt_screen_render( term->screen) ; */
+    /* vt_screen_render(term->screen); */
     vt_screen_visual(term->screen);
   }
 
